// BankSimulator
// Bank.h

#import <objectbase.h>
#import <objectbase/SwarmObject.h>
#import <random.h>
#import <space.h>

@interface Bank: SwarmObject {

  // STATIC BANK PARAMETERS
  unsigned short bankID;               // ID of the bank
  double riskAv;                       // Risk aversion of the bank
  double divFac;                       // Preference for diversification
  unsigned short basApp;               // Basel II aproach
  double targCapRat;                   // Target Capital Ratio
  double margin;                       // Margin for the interest
  double costInc;                      // Cost/ Income-Ratio
  double costRat;                      // Cost of Rating Systems per period
  double memStrength;                  // Memory Strength
  double Gini;                         // Gini coefficients of the rating segments                
  double PDMean;                       // PD of the customer portfolio
  double PDSigma;                      // Variance of the customer portfolio
  id<NormalDist> PDDist;               // Distribution of dropouts in the population

  // DYNAMIC BANK PARAMETERS 
  <Array> exposure;                    // Exposure of customer portfolio in each rating segment
  <Array> RWA;                         // Risk weighted assets in each rating segment
  double profit;                       // Profit in each rating segment
  double totExp;                       // Total Exposure
  double totRWA;                       // Total RWA
  double totProf;                      // Total profit
  double capRat;                       // Capital Ratio
  double capital;                      // Core Capital
  double intRat;                       // Interest Rate, i.e., (provisions for drop-outs + refinancing cost) * margin
  double intCap;                       // Interest Rate for Refinancing from Capital Markets
  <Array> dropCust;                    // Ex-post drop-out rate for each rating segment
  <Array> lossCust;                    // Total losses from drop-outs of customers
  double custSize;

  id<Grid2d> world;
  unsigned short selMarker;
  double intRate;


  /// PARAMETER SET BY MODEL.SETUP
  unsigned short reportingBankID;
}

+createBegin: (id) aZone;
-createEnd;

-initialize;

-setID: (unsigned short) idnum;
-(unsigned short) getID;

-setRiskAv: (double) R;
-(double) getRiskAv;

-setDivFac: (double) xi;
-(double) getDivFac;

-setBasApp: (unsigned short) A;
-(unsigned short) getBasApp;

-setTargCapRat: (double) t;
-(double) getTargCapRat;

-setMargin: (double) m;
-(double) getMargin;

-setCostInc: (double) CI;
-(double) getCostInc;

-setCostRat: (double) CR;
-(double) getCostRat;

-setMemStrength: (double) MS;
-(double) getMemStrength;

-setTotExp: (double) TE;
-(double) getTotExp;

-setTotRWA: (double) TRWA;
-(double) getTotRWA;

-setTotProf: (double) TP;
-(double) getTotProf;

-setCapRat: (double) CR;
-(double) getCapRat;

-setCapital: (double) C;
-(double) getCapital;

-setIntCap: (double) IC;
-(double) getIntCap;

-setGin: (double) G;
-(double) getGin;

-setIntRat: (double) IR;
-(double) getIntRat;

-setPDMean: (double) PD;
-(double) getPDMean;

-setPDSigma: (double) PDS;
-(double) getPDSigma;



-updateRWA;
-updateCapRat;

-updateDefaults;
-updateProfits;

-drawSelfOn: (id<Raster>)r;
-setWorld: (id<Grid2d>)w;
-setCustSize: (double) CS;

-setSelMarker: (unsigned short) x;
-(unsigned short) getSelMarker;


@end



