// BANK SIMULATOR
// ModelSwarm.m				

#import "ModelSwarm.h"
#import <simtools.h>
#import <random.h>

@implementation ModelSwarm

+createBegin: aZone {  
  ModelSwarm * obj;
  obj = [super createBegin: aZone];
  return obj;
}

-createEnd {
  [ObjectLoader load: self fromFileNamed: "model.setup"];
  // [ObjectLoader load: self fromFileNamed: "bank.setup"];
  return [super createEnd];
}

-buildObjects {
  id aBank;
  int i;
  printf("0\n");
  [randomGenerator setStateFromSeed: randomSeed];
  time = 0;

  bankList = [List create: self]; // create list of banks in the sector; using Zone prevents memory leaks
  //[bankList setCount: numBanks];
  capDist= [NormalDist createWithDefaults: [self getZone]];
  targCapDist= [NormalDist createWithDefaults: [self getZone]]; 
  riskAvDist= [NormalDist createWithDefaults: [self getZone]];
  divFacDist= [NormalDist createWithDefaults: [self getZone]];
  marginDist= [NormalDist createWithDefaults: [self getZone]]; 
  costIncDist= [NormalDist createWithDefaults: [self getZone]];
  costRatDist= [NormalDist createWithDefaults: [self getZone]];
  intRatDist=[NormalDist createWithDefaults: [self getZone]];
  memStrengthDist= [NormalDist createWithDefaults: [self getZone]];
  expoDist=[NormalDist createWithDefaults: [self getZone]];
  ginDist=[NormalDist createWithDefaults: [self getZone]];

  for (i = 0; i < numBanks; i++) { // populate the bidderList with initialized bidders
    aBank = [Bank createBegin: self];
    [aBank createEnd];
    [aBank setID: i];
    while([aBank getCapital]<=0)
      [aBank setCapital: [capDist getSampleWithMean: capMean withVariance: capSigma]];
    while ([aBank getTargCapRat]<=0)
      [aBank setTargCapRat: [targCapDist getSampleWithMean: targCapMean withVariance: targCapSigma]];
    while ([aBank getRiskAv]<=0)
      [aBank setRiskAv: [riskAvDist getSampleWithMean: riskAvMean withVariance: riskAvSigma]];
    while ([aBank getDivFac]<=0)
      [aBank setDivFac: [divFacDist getSampleWithMean: divFacMean withVariance: divFacSigma]];
    while ([aBank getMargin]<=0)
      [aBank setMargin: [marginDist getSampleWithMean: marginMean withVariance: marginSigma]];
    while ([aBank getCostInc]<=0)
      [aBank setCostInc: [costIncDist getSampleWithMean: costIncMean withVariance: costIncSigma]];
    while([aBank getCostRat]<=0)
      [aBank setCostRat: [costRatDist getSampleWithMean: costRatMean withVariance: costRatSigma]];
    while([aBank getIntRat]<=0)
      [aBank setIntRat: [intRatDist getSampleWithMean: intRatMean withVariance: intRatSigma]];
   while ([aBank getMemStrength]<=0)
      [aBank setMemStrength: [memStrengthDist getSampleWithMean: memStrengthMean withVariance: memStrengthSigma]];
    while ([aBank getTotExp]<=0)
      [aBank setTotExp: [expoDist getSampleWithMean: expoMean withVariance: expoSigma]];
    while ([aBank getGin]<=0 || [aBank getGin]>=1)
      [aBank setGin: [ginDist getSampleWithMean: ginMean withVariance: ginSigma]];

    [aBank setBasApp: ([uniformDblRand getDoubleWithMin:0 withMax: 1]<basAppRat)]; 
    [aBank setIntCap: intCap];
    [aBank updateRWA];
    [aBank updateCapRat];
    [aBank setPDMean: PDMean];
    [aBank setPDSigma: PDSigma];

    [aBank initialize];
    [bankList addLast: aBank];
  }
  customerSize=0;
  for (i=0; i<numBanks; i++) {
    customerSize=customerSize+[[bankList atOffset: i] getTotExp];
  }
  printf("Customer Size =%f\n",customerSize);
  world=  [Grid2d create: self setSizeX: 100 Y: 100];

  return self;
}


-getBankList {
  return bankList;
}

-(id<Grid2d>) getWorld {
   return world;
}

-(int) getTime {
  return time;
}

-step {
  printf("1\n");
  [self reportBankVars];  
  printf("2\n");
  [self updateBankParameters];
  printf("3\n");
  [self updateBankList];
  printf("4\n");
  [self updateAverages];
  printf("5\n");
  [self updateCustPortfolio];    
    time++;
    return self;
}


-updateBankParameters {
  int i;
  id aBank;

  for (i=0; i<numBanks; i++){
    aBank=[bankList atOffset: i];
    [aBank updateDefaults];
    [aBank updateProfits];
    [aBank updateRWA];
    [aBank updateCapRat];
  }

  return self;
}

-updateBankList {
  int i;
  id aBank, newBank;

  for (i=0; i<numBanks; i++) {
    aBank=[bankList atOffset: i];
    if ([aBank getCapital]<capTreshold || [aBank getCapRat]<capRatTreshold) {
      [[bankList atOffset: i]drop];
      newBank = [Bank createBegin: self];
      [newBank createEnd];
      [newBank setID: i];
      while([newBank getCapital]<=0)
	[newBank setCapital: [capDist getSampleWithMean: capMean withVariance: capSigma]];
      while ([newBank getTargCapRat]<=0)
	[newBank setTargCapRat: [targCapDist getSampleWithMean: targCapMean withVariance: targCapSigma]];
      while ([newBank getRiskAv]<=0)
	[newBank setRiskAv: [riskAvDist getSampleWithMean: riskAvMean withVariance: riskAvSigma]];
      while ([newBank getDivFac]<=0)
	[newBank setDivFac: [divFacDist getSampleWithMean: divFacMean withVariance: divFacSigma]];
      while ([newBank getMargin]<=0)
	[newBank setMargin: [marginDist getSampleWithMean: marginMean withVariance: marginSigma]];
      while ([newBank getCostInc]<=0)
	[newBank setCostInc: [costIncDist getSampleWithMean: costIncMean withVariance: costIncSigma]];
      while([newBank getCostRat]<=0)
	[newBank setCostRat: [costRatDist getSampleWithMean: costRatMean withVariance: costRatSigma]];
      while([newBank getIntRat]<=0)
	[newBank setIntRat: [intRatDist getSampleWithMean: intRatMean withVariance: intRatSigma]];
      while ([newBank getMemStrength]<=0)
	[newBank setMemStrength: [memStrengthDist getSampleWithMean: memStrengthMean withVariance: memStrengthSigma]];
     while ([newBank getTotExp]<=0)
	[newBank setTotExp: [expoDist getSampleWithMean: expoMean withVariance: expoSigma]];
     while ([newBank getGin]<=0 || [newBank getGin]>=1)
	[newBank setGin: [ginDist getSampleWithMean: ginMean withVariance: ginSigma]];     
      [newBank setBasApp: ([uniformDblRand getDoubleWithMin:0 withMax: 1]<basAppRat)]; 
      [newBank setIntCap: intCap];
      [newBank updateRWA];
      [newBank updateCapRat];
      [newBank setPDMean: PDMean];
      [newBank setPDSigma: PDSigma];
      [newBank initialize];
      [bankList addLast: newBank];
    }


  }

  return self;
}

-reportBankVars {

    printf("n\n********** TIME %d **********\n\n",time);
    // printf("Variables of Bank %d\n", reportingBankID);
    // printf("Capital: %f \t targCap: %f \t riskAv: %f\n", [[bankList atOffset: reportingBankID] getCapital], [[bankList atOffset: reportingBankID] getTargCapRat], [[bankList atOffset: reportingBankID] getRiskAv]);
    // printf ("Div. Factor: %f \t margin: %f \t CI-Ratio: %f\n",[[bankList atOffset: reportingBankID] getDivFac], [[bankList atOffset: reportingBankID] getMargin], [[bankList atOffset: reportingBankID] getCostInc]);
    // printf("Cost of RS: %f \t MemStrength: %f \t CapM-Interest: %f\n", [[bankList atOffset: reportingBankID] getCostRat], [[bankList atOffset: reportingBankID] getMemStrength], [[bankList atOffset: reportingBankID] getIntCap]);
    // printf("Tot Exp: %f \t Tot RWA: %f \t CapRat: %f\n",[[bankList atOffset: reportingBankID] getTotExp], [[bankList atOffset: reportingBankID] getTotRWA], [[bankList atOffset: reportingBankID] getCapRat]); 
    // printf("Bas II-App: %d \t\t Gini: %f \t interestRate:%f\n", [[bankList atOffset: reportingBankID]getBasApp], [[bankList atOffset: reportingBankID] getGin],[[bankList atOffset: reportingBankID] getIntRat]);
    return self;
}

-(int) getNumBanks {
  return numBanks;
}


-updateAverages{
  int i;
  double totalGini=0.0;
  double totalIR=0.0;
  id aBank;

  numSTA=0;
  numFIRB=0;

  for (i=0; i<numBanks; i++) {
    aBank=[bankList atOffset: i];
    printf("i=%d",i);
    printf("uA1\n");
    totalGini+=[aBank getGin];
    printf("uA2\n");
    totalIR=totalIR+[aBank getIntRat];

    //also update the number of STA/ FIRB-banks here
    if ([aBank getBasApp]==0)
      numSTA++;
    else 
      numFIRB++;
  }

  averageGini=totalGini/numBanks;
  averageIR=totalIR/numBanks;
  return self;
}    
  
-(double) getAverageGini {
  return averageGini;
}

-(double) getAverageIR {
  return averageIR;
}

-(double) getPropSTA { // Proportion of STA-banks
  return (double) numSTA/(double) numBanks;
}



-updateCustPortfolio {
  double tempCPF;
  id aBank, winBank;
  int i, j;
  double addExp;
  double tempIntRat;
  int tempWinID;


  tempCPF = customerSize;
  for (i=0; i<numBanks; i++) {// Initialize the winning marker
    [[bankList atOffset: i] setSelMarker: 0];
        tempCPF=tempCPF-[[bankList atOffset: i] getTotExp]; // Total Demand for Credits from Customers
  }

  // First of all update the exposure by simply shifting the money from capital to exposure

 
  for (j=0; j<numBanks; j++){
    tempIntRat=1000;
    for (i=0; i<numBanks; i++) {// determine the bank with the smallest interest rate, so far not chosen
      aBank=[bankList atOffset: i];
      if ([aBank getIntRat]<tempIntRat && [aBank getSelMarker]==0){
	tempWinID=i;
	tempIntRat=[aBank getIntRat];
      }
    }
    winBank=[bankList atOffset: tempWinID];
    [winBank setSelMarker: 1];
    addExp=[winBank getTotExp] *([winBank getCapital] - [winBank getTotRWA]*[winBank getTargCapRat])/([winBank getTotExp]+[winBank getTotRWA]*[winBank getTargCapRat]);
    // nextLine to be merged with the previous one
    // addExp=[aBank getCapital]*[aBank getTargCapRat]*[aBank getTotExp]/[aBank getTotRWA]-[aBank getTotExp];
    //if (addExp>0){ // Only put out new credits if exposure needs to be added
      if (addExp>tempCPF) { // Do not take more new credits than customers are available 
	addExp=tempCPF;
	tempCPF=0;
      }
      if (0<addExp && addExp<tempCPF){
	tempCPF=tempCPF-addExp;// if you need less customers than available then update all of themtake all you need
      } 
      // printf("B: %d IR: %f G: %f C: %f XP: %f RW: %f\n",tempWinID, tempIntRat, [winBank getGin], [winBank getCapital], [winBank getTotExp], [winBank getTotRWA]);
      // printf("    t: %f q: %f addExp: %f BasApp: %d\n",[winBank getTargCapRat],[winBank getCapRat],addExp, [winBank getBasApp]); 
      [winBank setCapital: [aBank getCapital]-addExp];
      [winBank setTotExp: [winBank getTotExp]+addExp];
      [winBank updateCapRat];
      if (addExp>0) 
	printf("Bank %d adds %f->Outst. Exp: %f\n",tempWinID,addExp,tempCPF);
    }

  printf("\n************************\n");
  printf("*** Average Gini: %f\n",averageGini);
  printf("*** Average IntRat: %f\n", averageIR);
  printf("*************************\n");
  return self;
}

@end
