
// ObserverSwarm.m

#import "ObserverSwarm.h"
#import "ModelSwarm.h"
// #import "Bank.h"
#import <collections.h>
#import <objectbase.h>
#import <analysis.h>
#import <gui.h>

@implementation ObserverSwarm

+createBegin: aZone { //CHANGE ! I TOOK (id) aZone OUT!!

 ObserverSwarm * obj;
 id <CustomProbeMap> probeMap;

 // superClass createBegin to allocate ourselves
  obj = [super createBegin: aZone];

  // Fill in the relevant parameters (only 1 in our case)

  obj->displayFrequency = 1;

  // Build a customized probe map
  probeMap = [CustomProbeMap create: aZone
			     forClass: [self class]
			     withIdentifiers: "displayFrequency",
			     NULL];
  // Install the probeMap into the probeLibrary
  [probeLibrary setProbeMap: probeMap For: [self class]];
  return obj;
}

// create obejcts that we might need (in our case none, but you may want to override this)

-createEnd {
  return [super createEnd];
}

-_bankRasterDeath_ : caller {
  [bankRaster drop];
  bankRaster=nil;
  return self;
}


-buildObjects {
  [super buildObjects];

  modelSwarm = [ModelSwarm create: self];
  

  CREATE_ARCHIVED_PROBE_DISPLAY (modelSwarm);
  CREATE_ARCHIVED_PROBE_DISPLAY (self);
  [modelSwarm buildObjects];
  [controlPanel setStateStopped];

  averGin=[EZGraph createBegin: [self getZone]];
  SET_WINDOW_GEOMETRY_RECORD_NAME (averGin);
  [averGin setTitle: "Average of Gini Coefficients"];
  [averGin setAxisLabelsX: "Time" Y:"avGin"];
  [averGin createEnd];

  [averGin createSequence: "average Gini" withFeedFrom: modelSwarm andSelector: M(getAverageGini)];

  averIR=[EZGraph createBegin: [self getZone]];
  SET_WINDOW_GEOMETRY_RECORD_NAME (averIR);
  [averIR setTitle: "Average of Interest Rates"];
  [averIR setAxisLabelsX: "Time" Y:"avIR"];
  [averIR createEnd];

  [averIR createSequence: "avIR" withFeedFrom: modelSwarm andSelector: M(getAverageIR)];
  
  propSTA=[EZGraph createBegin: [self getZone]];
  SET_WINDOW_GEOMETRY_RECORD_NAME (propSTA);
  [propSTA setTitle: "Proportion of STA-banks"];
  [propSTA setAxisLabelsX: "Time" Y:"propSTA"];
  [propSTA createEnd];

  [propSTA createSequence: "propSTA" withFeedFrom: modelSwarm andSelector: M(getPropSTA)];


  colormap= [Colormap create: self];
  [colormap setColor: 0 ToName: "Red"];
  [colormap setColor: 1 ToRed: 0 Green: 63 Blue: 63];

  bankRaster = [ZoomRaster createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (bankRaster);
  bankRaster= [bankRaster createEnd];

  [bankRaster enableDestroyNotification: self
	      notificationMethod: @selector (_bankRasterDeath_:)];
  [bankRaster setColormap: colormap];
  [bankRaster setZoomFactor: 4];
  [bankRaster setWidth: 150 Height: 150];
  [bankRaster setWindowTitle: "Banking Sector"];
  [bankRaster pack];

  // Now create a Value2dDisplay : that is a special object that will display
  // arbitrary 2d value arrays on a given Raster widget

  // bankDisplay = [Object2dDisplay create: self
  //				 setDisplayWidget: bankRaster
  //			 setDiscrete2dToDisplay: [modelSwarm getWorld]
  //			 setDisplayMessage: M(drawSelfOn)];
  //[bankDisplay setObjectCollection: [modelSwarm getBankList]];

  // Also allow the world raster to send mouse clicks to the bank display
  // This allows the user to right-click on the display to probe the banks

  // [bankRaster setButton: ButtonRight 
  //	      Client: bankDisplay
  //      Message: M(makeProbeAtX:Y:)];

  return self;
}

-_update_{
  int i, numBanks;
  double tempExp, tempCap;
  int tempBasApp;
  [bankRaster erase];
  numBanks=[modelSwarm getNumBanks];
  if(bankRaster){
    for (i=0; i<numBanks; i++) {
      tempExp = [[[modelSwarm getBankList] atOffset: i] getTotExp];
      tempCap = [[[modelSwarm getBankList] atOffset: i] getCapital];   
      tempBasApp=[[[modelSwarm getBankList] atOffset: i] getBasApp];
      [bankRaster fillRectangleX0: tempExp/1000+10 Y0: tempCap/500+10 X1: tempExp/1000+11 Y1: tempCap/500+11 Color: tempBasApp]; 
    }
    [bankRaster drawSelf];
    [actionCache doTkEvents];
  }
  return self;
}



-buildActions {
  [super buildActions];

  [modelSwarm buildActions];

  displayActions = [ActionGroup create: self];
  [displayActions createActionTo: self                message: M(_update_)];
  [displayActions createActionTo: probeDisplayManager message: M(update)];
  printf("Obs1\n");
  [displayActions createActionTo: averGin               message: M(step)];
  [displayActions createActionTo: averIR                message: M(step)];
  [displayActions createActionTo: propSTA               message: M(step)]; 
  [displayActions createActionTo: modelSwarm                  message: M(step)];
  [displayActions createActionTo: actionCache                 message: M(doTkEvents)];

  displaySchedule = [Schedule create: self setRepeatInterval: 1];
  [displaySchedule at: 0 createAction: displayActions];

  return self;
}

-activateIn: (id) swarmContext {
  [super activateIn: swarmContext];
  // [modelSwarm activateIn: self];  // necessary, if modelSwarm has activity schedule 
  [displaySchedule activateIn: self];
  return [self getSwarmActivity];
}

-graphBank: aBank {
  return self;
}



@end








